from django.core.management.base import BaseCommand

from rq import Queue, Worker
from redis.exceptions import ConnectionError

from django_rq import settings
from django_rq.queues import get_connection


class Command(BaseCommand):
    """
    RQ consumer
    """

    def handle(self, *args, **options):
        try:
            queues = [Queue(name, connection=get_connection(name)) for name in settings.QUEUES]
            for queue in queues:
                w = Worker([queue], connection=queue.connection)
                w.work(burst=settings.BURST)
        except ConnectionError as e:
            print(e)



from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.test.utils import override_settings

from rq.job import Job

from .queues import get_connection, get_queue


TEST_QUEUES = {
    'default': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'test': {
        'HOST': 'localhost',
        'PORT': 1,
        'DB': 1,
    }
}


class DjangoRQTest(TestCase):

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    def test_get_connection(self):
        """
        Test that get_connection returns the right connection based on 
        settings.RQ_QUEUES
        """
        config = TEST_QUEUES['default']
        connection = get_connection()
        connection_kwargs = connection.connection_pool.connection_kwargs
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

        config = TEST_QUEUES['test']
        connection = get_connection('test')
        connection_kwargs = connection.connection_pool.connection_kwargs
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

    @override_settings(RQ_QUEUES=TEST_QUEUES)
    def test_get_queue(self):
        # Test that get_queue use the right parameters for its connection
        config = TEST_QUEUES['default']
        queue = get_queue('default')
        connection_kwargs = queue.connection.connection_pool.connection_kwargs
        self.assertEqual(queue.name, 'default')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

        config = TEST_QUEUES['test']
        queue = get_queue('test')
        connection_kwargs = queue.connection.connection_pool.connection_kwargs
        self.assertEqual(queue.name, 'test')
        self.assertEqual(connection_kwargs['host'], config['HOST'])
        self.assertEqual(connection_kwargs['port'], config['PORT'])
        self.assertEqual(connection_kwargs['db'], config['DB'])

    @override_settings(RQ_QUEUES=None)
    def test_empty_queue_setting_raises_exception(self):
        # Raise an exception if RQ_QUEUES is not defined
        self.assertRaises(ImproperlyConfigured, get_connection)


from django.shortcuts import render

from rq import Worker

from .queues import get_connection, get_queue
from .settings import QUEUES


def stats(request):
    queues = []
    for name in QUEUES:
        queue = get_queue(name)
        connection = get_connection(name)
        all_workers = Worker.all(connection=connection)
        queue_workers = [worker for worker in all_workers if queue in worker.queues]
        stat = {
            'name': name,
            'jobs': queue.count,
            'workers': len(queue_workers)
        }
        queues.append(stat)
    context_data = { 'queues': queues }
    return render(request, 'django_rq/stats.html', context_data)


import redis
from rq import Queue


def get_connection(name='default'):
    """
    Returns a Redis connection to use based on parameters in settings.RQ_QUEUES
    """
    from .settings import QUEUES
    queue_config = QUEUES[name]
    return redis.Redis(host=queue_config['HOST'],
        port=queue_config['PORT'], db=queue_config['DB'])


def get_queue(name='default'):
    """
    Returns an rq Queue using parameters defined in ``RQ_QUEUES``
    """
    return Queue(name, connection=get_connection(name))


def enqueue(func, *args, **kwargs):
    '''
    A convenience function to put a job in the default queue. Usage::

    from django_rq import enqueue
    enqueue(func, *args, **kwargs)
    '''
    return get_queue().enqueue(func, *args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='django-rq',
    version='0.1.1',
    author='Selwin Ong',
    author_email='selwin.ong@gmail.com',
    packages=['django_rq'],
    url='https://github.com/ui/django-rq',
    license='MIT',
    description='A simple app that provides django integration for RQ (Redis Queue)',
    long_description=open('README.rst').read(),
    zip_safe=False,
    include_package_data=True,
    package_data = { '': ['README.rst'] },
    install_requires=['django', 'rq'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)